Class: Localhost::Authority

Inherits:
Object
  • Object
show all
Defined in:
lib/localhost/authority.rb

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(hostname = "localhost") ⇒ Authority



43
44
45
46
47
48
49
50
# File 'lib/localhost/authority.rb', line 43

def initialize(hostname = "localhost")
  @hostname = hostname
  
  @key = nil
  @name = nil
  @certificate = nil
  @store = nil
end

Class Method Details

.fetch(*args) ⇒ Object



30
31
32
33
34
35
36
37
38
39
40
41
# File 'lib/localhost/authority.rb', line 30

def self.fetch(*args)
  authority = self.new(*args)
  path = self.path
  
  unless authority.load(path)
    Dir.mkdir(path, 0700) unless File.directory?(path)
    
    authority.save(path)
  end
  
  return authority
end

.pathObject



26
27
28
# File 'lib/localhost/authority.rb', line 26

def self.path
  File.expand_path("~/.localhost")
end

Instance Method Details

#certificateObject



60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
# File 'lib/localhost/authority.rb', line 60

def certificate
  @certificate ||= OpenSSL::X509::Certificate.new.tap do |certificate|
    certificate.subject = self.name
    # We use the same issuer as the subject, which makes this certificate self-signed:
    certificate.issuer = self.name
    
    certificate.public_key = self.key.public_key
    
    certificate.serial = 1
    
    certificate.not_before = Time.now
    certificate.not_after = Time.now + (3600 * 24 * 365 * 10)
    
    extension_factory = OpenSSL::X509::ExtensionFactory.new
    extension_factory.subject_certificate = certificate
    extension_factory.issuer_certificate = certificate
    
    # Because we are using a self-signed root certificate, we also need to make it a "pseudo-CA".
    # https://security.stackexchange.com/questions/143061/does-openssl-refuse-self-signed-certificates-without-basic-constraints
    certificate.add_extension extension_factory.create_extension("basicConstraints", "CA:TRUE", true)
    certificate.add_extension extension_factory.create_extension("keyUsage", "keyCertSign, cRLSign, digitalSignature", true)
    certificate.add_extension extension_factory.create_extension("subjectKeyIdentifier", "hash")
    
    certificate.sign self.key, OpenSSL::Digest::SHA256.new
  end
end

#client_context(*args) ⇒ Object



109
110
111
112
113
114
115
116
117
118
119
120
121
# File 'lib/localhost/authority.rb', line 109

def client_context(*args)
  OpenSSL::SSL::SSLContext.new(*args).tap do |context|
    context.cert_store = self.store
    
    context.set_params(
      verify_mode: OpenSSL::SSL::VERIFY_PEER,
    )
    
    if context.respond_to? :verify_hostname=
      context.verify_hostname = false
    end
  end
end

#keyObject



52
53
54
# File 'lib/localhost/authority.rb', line 52

def key
  @key ||= OpenSSL::PKey::RSA.new(1024)
end

#load(path) ⇒ Object



123
124
125
126
127
128
129
130
131
132
133
134
# File 'lib/localhost/authority.rb', line 123

def load(path)
  if File.directory? path
    key_path = File.join(path, "#{@hostname}.key")
    return false unless File.exist?(key_path)
    @key = OpenSSL::PKey::RSA.new(File.read(key_path))
    
    certificate_path = File.join(path, "#{@hostname}.crt")
    @certificate = OpenSSL::X509::Certificate.new(File.read(certificate_path))
    
    return true
  end
end

#nameObject



56
57
58
# File 'lib/localhost/authority.rb', line 56

def name
  @name ||= OpenSSL::X509::Name.parse("O=Development/CN=#{@hostname}")
end

#save(path) ⇒ Object



136
137
138
139
140
141
142
143
144
145
146
# File 'lib/localhost/authority.rb', line 136

def save(path)
  File.write(
    File.join(path, "#{@hostname}.crt"),
    self.certificate.to_pem
  )
  
  File.write(
    File.join(path, "#{@hostname}.key"),
    self.key.to_pem
  )
end

#server_context(*args) ⇒ Object



94
95
96
97
98
99
100
101
102
103
104
105
106
107
# File 'lib/localhost/authority.rb', line 94

def server_context(*args)
  OpenSSL::SSL::SSLContext.new(*args).tap do |context|
    context.key = self.key
    context.cert = self.certificate
    
    context.session_id_context = "localhost"
    
    context.set_params
    
    if context.respond_to? :verify_hostname=
      context.verify_hostname = false
    end
  end
end

#storeObject

The certificate store which is used for validating the server certificate:



88
89
90
91
92
# File 'lib/localhost/authority.rb', line 88

def store
  @store ||= OpenSSL::X509::Store.new.tap do |store|
    store.add_cert(self.certificate)
  end
end